package com.ozs.web.controller.accountmanagment;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ozs.common.annotation.Log;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.core.domain.entity.SysRole;
import com.ozs.common.core.redis.RedisCache;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.exception.base.BaseException;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.ChineseToPingyin;
import com.ozs.common.utils.HttpClientUtil;
import com.ozs.common.utils.IdempotenceUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.file.FileUtils;
import com.ozs.common.utils.http.HttpUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.framework.config.ServerConfig;
import com.ozs.service.entity.*;
import com.ozs.service.entity.vo.*;
import com.ozs.service.service.*;
import com.ozs.service.utils.PublishClient;
import com.ozs.system.service.DataScoreUtil;
import com.ozs.system.service.ISysDeptService;
import com.ozs.system.service.ISysDictTypeService;
import com.ozs.web.core.config.CaneraConfig;
import com.ozs.web.core.util.CameraUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static net.sf.jsqlparser.parser.feature.Feature.update;

/**
 * <p>
 * 相机台账管理表 前端控制器
 * </p>
 *
 * @author ozs
 * @since 2023-02-17
 */
@RestController
@RequestMapping("/service/baseCameraManagement")
@Slf4j
public class BaseCameraManagementController extends BaseController {
    public static final String PATTERN = "^(\\d+.\\d{1,3}|\\d+)$";
    @Resource
    BaseCameraManagementService baseCameraManagementService;
    @Resource
    BaseDeviceDynamicManagementService baseDeviceDynamicManagementService;
    @Autowired
    private ISysDeptService deptService;
    @Autowired
    private MsgAlarmService msgAlarmService;
    @Autowired
    private BaseRailwayManagementService baseRailwayManagementService;
    @Autowired
    private ServerConfig serverConfig;
    @Autowired
    private CaneraConfig caneraConfig;
    @Autowired
    private IdempotenceUtils idempotenceUtils;
    @Autowired
    private DataScoreUtil dataScoreUtil;
    @Resource
    private BaseUserService baseUserService;
    @Autowired
    private RedisCache redisCache;
    @Autowired
    private ISysDictTypeService dictTypeService;
    @Autowired
    private CameraUtil cameraUtil;
    @Autowired
    private MqLogService mqLogService;


    /**
     * 所有相机列表+权限
     */
    @GetMapping("/selectCameraAll")
    @Log(title = "相机台账管理", businessType = BusinessType.SELECT)
    public AjaxResult selectCameraAll() {
        BaseCameraManagementVo baseCameraManagementVo = (BaseCameraManagementVo) dataScoreUtil.setDataScore(getUserId(), new BaseCameraManagementVo());
        List<BaseCameraManagementHomeVo> baseCameraManagements = baseCameraManagementService.selectCameraAll(baseCameraManagementVo);
        return AjaxResult.success(baseCameraManagements);
    }

    @PostMapping(value = "/homeGetInfo")
    @Log(title = "相机台账管理", businessType = BusinessType.SELECT)
    public AjaxResult homeGetInfo(@RequestBody BaseCameraManagementHomeVo baseCameraManagementHomeVo) {
        Integer isLock = baseCameraManagementHomeVo.getIsLock();
        Long id = baseCameraManagementHomeVo.getId();
        String cameraCode = baseCameraManagementHomeVo.getCameraCode();
        // isLock  报警是否解除 1已解除2未解除
        if (!StringUtils.isEmpty(isLock) && isLock == 1 && !StringUtils.isEmpty(id)) {
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
            LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                queryWrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
            SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
            baseCameraManagement.setDeptName(sysDept.getDeptName());
            baseCameraManagement.setRailwayName(baseRailwayManagement.getRailwayName());
            String mils = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            baseCameraManagement.setInstallMiles(mils);
            if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile())) {
                String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
                baseCameraManagement.setEndMiles(end);
            }
            if (!ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
                String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
                baseCameraManagement.setBeginMiles(begin);
            }
            BaseDeviceDynamicManagement deviceDynamicManagement = baseDeviceDynamicManagementService.getOne(new LambdaQueryWrapper<BaseDeviceDynamicManagement>()
                    .eq(BaseDeviceDynamicManagement::getCameraCode, cameraCode));
            baseCameraManagement.setIsDisable(deviceDynamicManagement.getIsDisable());
            baseCameraManagement.setElectricity(deviceDynamicManagement.getElectricity()+"%");
            List<MqLog> list = mqLogService.list(new LambdaQueryWrapper<MqLog>()
                    .eq(MqLog::getCameraCode, cameraCode));
            if (!ObjectUtils.isEmpty(list)) {
                baseCameraManagement.setHeartTime(list.get(0).getCreateTime());
            }
            // 将字符串转换为浮点数
            double installLatitude = Double.parseDouble(baseCameraManagement.getInstallLatitude());
            double installLongitude = Double.parseDouble(baseCameraManagement.getInstallLongitude());
            // 创建 DecimalFormat 对象，指定格式
            DecimalFormat decimalFormat = new DecimalFormat("#.00");
            // 格式化数字
            baseCameraManagement.setInstallLatitude(decimalFormat.format(installLatitude));
            baseCameraManagement.setInstallLongitude(decimalFormat.format(installLongitude));
            return AjaxResult.success(baseCameraManagement);
        } else if (!StringUtils.isEmpty(isLock) && isLock == 2 && !StringUtils.isEmpty(cameraCode)) {
            MsgAlarm msgAlarm = msgAlarmService.selectByCameraCode(cameraCode);
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile()) && !ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
                String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
                String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
                String endMile = begin + "-" + end;
                msgAlarmResp.setBeginMile(endMile);
            }
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            double installLatitude = Double.parseDouble(baseCameraManagement.getInstallLatitude());
            double installLongitude = Double.parseDouble(baseCameraManagement.getInstallLongitude());
            // 创建 DecimalFormat 对象，指定格式
            DecimalFormat decimalFormat = new DecimalFormat("#.00");
            // 格式化数字
            msgAlarmResp.setInstallLongitude(decimalFormat.format(installLongitude) + "," + decimalFormat.format(installLatitude));
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarmResp.setImageUrls(objects);
            BaseDeviceDynamicManagement deviceDynamicManagement = baseDeviceDynamicManagementService.getOne(new LambdaQueryWrapper<BaseDeviceDynamicManagement>()
                    .eq(BaseDeviceDynamicManagement::getCameraCode, cameraCode));
            msgAlarmResp.setIsDisable(deviceDynamicManagement.getIsDisable());
            msgAlarmResp.setElectricity(deviceDynamicManagement.getElectricity()+"%");
            List<MqLog> list = mqLogService.list(new LambdaQueryWrapper<MqLog>()
                    .eq(MqLog::getCameraCode, cameraCode));
            if (!ObjectUtils.isEmpty(list)) {
                msgAlarmResp.setHeartTime(list.get(0).getCreateTime());
            }
            return AjaxResult.success(msgAlarmResp);
        }
        return null;
    }

    /**
     * 相机台账管理分页
     *
     * @param baseCameraManagementVo
     * @return
     */
    @PostMapping("/list")
    @Log(title = "相机台账管理", businessType = BusinessType.SELECT)
    public AjaxResult selectBaseAccountManagement(@RequestBody BaseCameraManagementVo baseCameraManagementVo) {
        baseCameraManagementVo = (BaseCameraManagementVo) dataScoreUtil.setDataScore(getUserId(), baseCameraManagementVo);
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getBeginMile())) {
            baseCameraManagementVo.setBeginMile(baseCameraManagementVo.getBeginMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getEndMile())) {
            baseCameraManagementVo.setEndMile(baseCameraManagementVo.getEndMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getInstallMile())) {
            baseCameraManagementVo.setInstallMile(baseCameraManagementVo.getInstallMile() * 1000);
        }
        IPage<BaseCameraManagement> page = baseCameraManagementService.queryPage(baseCameraManagementVo);
        page.setTotal(page.getTotal());
        page.setCurrent(page.getCurrent());
        page.setPages(page.getPages());
        List<String> roleKey = baseUserService.getRoleName(getUserId());
        BaseUser baseUser = baseUserService.getUser(getUserId());
        if (!ObjectUtils.isEmpty(page) && page.getRecords().size() > 0) {
            List<BaseCameraManagement> dto1 = page.getRecords().stream().map(o -> {
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(o.getRailwayCode())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, o.getRailwayCode());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
                SysDept sysDept = deptService.selectDeptById(o.getDeptId());
                o.setDeptName(sysDept.getDeptName());
                if (!StringUtils.isEmptySunhh(baseRailwayManagement) && !StringUtils.isEmptySunhh(baseRailwayManagement.getRailwayName())) {
                    o.setRailwayName(baseRailwayManagement.getRailwayName());
                }
                String mils = AppendUtils.stringAppend(o.getInstallMile());
                o.setInstallMiles(mils);
                if (!ObjectUtils.isEmpty(o.getEndMile())) {
                    String end = AppendUtils.stringAppend(o.getEndMile());
                    o.setEndMiles(end);
                }
                if (!ObjectUtils.isEmpty(o.getBeginMile())) {
                    String begin = AppendUtils.stringAppend(o.getBeginMile());
                    o.setBeginMiles(begin);
                }
                if (roleKey.size() > 0 || "admin".equals(baseUser.getUserName())) {
                    o.setIsStartRecording(1);
                    for (String s : roleKey) {
                        if ("admin".equals(s) || "admin".equals(baseUser.getUserName())) {
                            o.setIsStartRecording(1);
                            break;
                        } else {
                            o.setIsStartRecording(2);
                        }
                    }
                }
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }

    /**
     * 删除相机台账管理信息
     *
     * @param accountManagementIds
     * @return
     */
    @RequestMapping("/remove/{accountManagementIds}")
    @Log(title = "相机台账管理", businessType = BusinessType.DELETE)
    @Transactional
    public AjaxResult removeAccountManagement(@PathVariable List<Long> accountManagementIds) {
        for (Long accountManagementId : accountManagementIds) {
            LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(accountManagementId);
            if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
                queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, baseCameraManagement.getCameraCode());
            }
            baseDeviceDynamicManagementService.remove(queryWrapper);
            baseCameraManagementService.removeById(accountManagementId);
        }
        return AjaxResult.success();
    }

    /**
     * 新增相机台账管理信息
     *
     * @param baseCameraManagement
     * @return
     */
    @PostMapping("/saveAccountManagement")
    @Transactional
    @Log(title = "相机台账管理", businessType = BusinessType.INSERT)
    public AjaxResult saveAccountManagement(@RequestBody BaseCameraManagement baseCameraManagement) {
        try {
            String idempotence = idempotenceUtils.getIdempotence(baseCameraManagement.getCameraCode());
            if (StringUtils.isNotEmpty(idempotence)) {
                return error("正在执行,请稍等");
            }
            idempotenceUtils.createIdempotence(baseCameraManagement.getCameraCode());
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, baseCameraManagement.getCameraCode());
            }
            BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(lw);
            if (!ObjectUtils.isEmpty(cameraManagement)) {
                return error("相机台账管理信息相机编码已存在,不能重复添加");
            }
            LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
                queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, baseCameraManagement.getCameraCode());
            }
            BaseDeviceDynamicManagement one = baseDeviceDynamicManagementService.getOne(queryWrapper);
            if (!ObjectUtils.isEmpty(one)) {
                return error("设备动态管理表该相机编码已存在,不能重复添加");
            }
            if (!ObjectUtils.isEmpty(baseCameraManagement.getBeginMiles())) {
                if (!baseCameraManagement.getBeginMiles().matches(PATTERN)) {
                    return error("监控范围开始里程位置填写格式不正确");
                } else {
                    BigDecimal begin = new BigDecimal(baseCameraManagement.getBeginMiles());
                    baseCameraManagement.setBeginMile(begin.multiply(BigDecimal.valueOf(1000)).intValue());
                }
            }
            if (!baseCameraManagement.getInstallMiles().matches(PATTERN)) {
                return error("监控相机安装里程位置填写格式不正确");
            } else {
                BigDecimal install = new BigDecimal(baseCameraManagement.getInstallMiles());
                baseCameraManagement.setInstallMile(install.multiply(BigDecimal.valueOf(1000)).intValue());
            }
            if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMiles())) {
                if (!baseCameraManagement.getEndMiles().matches(PATTERN)) {
                    return error("监控范围结束里程位置填写格式不正确");
                } else {
                    BigDecimal end = new BigDecimal(baseCameraManagement.getEndMiles());
                    baseCameraManagement.setEndMile(end.multiply(BigDecimal.valueOf(1000)).intValue());
                }
            }
            baseCameraManagement.setCreateBy(getUserId());
            baseCameraManagement.setUpdateBy(getUserId());
            BaseDeviceDynamicManagement baseDynamicManagement = new BaseDeviceDynamicManagement();
            baseDynamicManagement.setCameraCode(baseCameraManagement.getCameraCode());
            baseDynamicManagement.setUpdateBy(getUserId());
            baseDynamicManagement.setCreateBy(getUserId());
            List<SysDictData> data = dictTypeService.selectDictDataByType("management_transcribe_switch");
            if (data.get(0).getDictValue().equals("true")) {
                String start = HttpUtils.sendGet(CameraUtil.startRecording(baseCameraManagement.getCameraCode(), baseCameraManagement.getChannel()));
                if (start.startsWith(baseCameraManagement.getCameraCode() + "/" + baseCameraManagement.getChannel())) {
                    baseCameraManagement.setStartRecording(1);
                    baseCameraManagement.setTaskId(start);
                } else {
                    baseCameraManagement.setStartRecording(2);
                }
            }
            if (baseCameraManagementService.save(baseCameraManagement) && baseDeviceDynamicManagementService.save(baseDynamicManagement)) {
                return AjaxResult.success();
            } else {
                return AjaxResult.error();
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            return AjaxResult.error();
        } finally {
            idempotenceUtils.deleteIdempotence(baseCameraManagement.getCameraCode());
        }
    }

    /**
     * 修改相机台账管理信息
     *
     * @param baseCameraManagement
     * @return
     */
    @PostMapping("/editAccountManagement")
    @Log(title = "相机台账管理", businessType = BusinessType.UPDATE)
    public AjaxResult editAccountManagement(@RequestBody BaseCameraManagement baseCameraManagement) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
            lw.eq(BaseCameraManagement::getCameraCode, baseCameraManagement.getCameraCode());
        }
        BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(lw);
        if (!ObjectUtils.isEmpty(cameraManagement) && !cameraManagement.getId().equals(baseCameraManagement.getId())) {
            return error("相机编码已存在,不能进行修改操作");
        }
        if (!ObjectUtils.isEmpty(baseCameraManagement.getBeginMiles())) {
            if (!baseCameraManagement.getBeginMiles().matches(PATTERN)) {
                return error("监控范围开始里程位置填写格式不正确");
            } else {
                BigDecimal begin = new BigDecimal(baseCameraManagement.getBeginMiles());
                baseCameraManagement.setBeginMile(begin.multiply(BigDecimal.valueOf(1000)).intValue());
            }
        } else {
            baseCameraManagement.setBeginMile(null);
        }
        if (!baseCameraManagement.getInstallMiles().matches(PATTERN)) {
            return error("监控相机安装里程位置填写格式不正确");
        } else {
            BigDecimal install = new BigDecimal(baseCameraManagement.getInstallMiles());
            baseCameraManagement.setInstallMile(install.multiply(BigDecimal.valueOf(1000)).intValue());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMiles())) {
            if (!baseCameraManagement.getEndMiles().matches(PATTERN)) {
                return error("监控范围结束里程位置填写格式不正确");
            } else {
                BigDecimal end = new BigDecimal(baseCameraManagement.getEndMiles());
                baseCameraManagement.setEndMile(end.multiply(BigDecimal.valueOf(1000)).intValue());
            }
        } else {
            baseCameraManagement.setEndMile(null);
        }
        BaseCameraManagement management = baseCameraManagementService.getById(baseCameraManagement.getId());
        baseCameraManagement.setUpdateBy(getUserId());
        baseCameraManagement.setCreateBy(management.getCreateBy());
        baseCameraManagement.setUpdateTime(new Date());
        LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        if (!ObjectUtils.isEmpty(management.getCameraCode())) {
            queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, management.getCameraCode());
        }
        BaseDeviceDynamicManagement one = baseDeviceDynamicManagementService.getOne(queryWrapper);
        if (ObjectUtils.isEmpty(one)) {
            return error("设备动态管理表没有该相机编码");
        }
        one.setCameraCode(baseCameraManagement.getCameraCode());
        LambdaUpdateWrapper<BaseCameraManagement> wrapper = new LambdaUpdateWrapper<>();
        wrapper.eq(BaseCameraManagement::getId, baseCameraManagement.getId());
        if (baseCameraManagementService.update(baseCameraManagement, wrapper) && baseDeviceDynamicManagementService.updateById(one)) {
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    @GetMapping(value = "/{id}")
    @Log(title = "相机台账管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getInfo(@PathVariable Long id) {
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
        LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
            queryWrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
        SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
        baseCameraManagement.setDeptName(sysDept.getDeptName());
        baseCameraManagement.setRailwayName(baseRailwayManagement.getRailwayName());

        Double install = (double) baseCameraManagement.getInstallMile() / 1000;
        String[] split = install.toString().split("\\.");
        int number = Integer.parseInt(split[1]);
        if (number == 0) {
            baseCameraManagement.setInstallMiles(split[0]);
        } else {
            baseCameraManagement.setInstallMiles(install.toString());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile())) {
            Double end = (double) baseCameraManagement.getEndMile() / 1000;
            String[] ends = end.toString().split("\\.");
            int numbers = Integer.parseInt(ends[1]);
            if (numbers == 0) {
                baseCameraManagement.setEndMiles(ends[0]);
            } else {
                baseCameraManagement.setEndMiles(end.toString());
            }
        }
        if (!ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
            Double begin = (double) baseCameraManagement.getBeginMile() / 1000;
            String[] begins = begin.toString().split("\\.");
            int numberss = Integer.parseInt(begins[1]);
            if (numberss == 0) {
                baseCameraManagement.setBeginMiles(begins[0]);
            } else {
                baseCameraManagement.setBeginMiles(begin.toString());
            }
        }
        String mils = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
        baseCameraManagement.setInstalls(mils);
        if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile()) && !ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
            String en = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            String beg = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            baseCameraManagement.setBeginString(beg);
            baseCameraManagement.setEndString(en);
            baseCameraManagement.setBegins(beg + "~" + en);
        }
        BaseUser baseUser = baseUserService.getUser(baseCameraManagement.getCreateBy());
        baseCameraManagement.setCreateBy(baseUser.getUserName());
        BaseUser user = baseUserService.getUser(baseCameraManagement.getUpdateBy());
        baseCameraManagement.setUpdateBy(user.getUserName());
        return AjaxResult.success(baseCameraManagement);
    }

    @GetMapping("/getDate/{cameraCode}")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult getDate(@PathVariable String cameraCode) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(cameraCode)) {
            lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        }
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
        LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
            wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
        if (ObjectUtils.isEmpty(baseRailwayManagement)) {
            return error("线路表中没有该线路编码:" + baseCameraManagement.getRailwayCode() + "的数据");
        }
        LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
            queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, baseCameraManagement.getCameraCode());
        }
        BaseDeviceDynamicManagement baseDeviceDynamicManagement = baseDeviceDynamicManagementService.getOne(queryWrapper);
        if (ObjectUtils.isEmpty(baseDeviceDynamicManagement)) {
            return error("设备动态管理表中没有该相机编码:" + baseCameraManagement.getCameraCode() + "的数据");
        }
        LambdaQueryWrapper<MsgAlarm> wrapper1 = new LambdaQueryWrapper<MsgAlarm>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
            wrapper1.eq(MsgAlarm::getCameraCode, baseCameraManagement.getCameraCode());
            wrapper1.orderByDesc(MsgAlarm::getAlarmTime);
        }
        List<MsgAlarm> list1 = msgAlarmService.list(wrapper1);
        List<Object> list = new ArrayList<>();
        list.add(baseRailwayManagement.getRailwayName());
        list.add(AppendUtils.stringAppend(baseCameraManagement.getInstallMile()));
        list.add(baseCameraManagement.getLineDir());
        list.add(baseCameraManagement.getCameraCode());
        list.add(baseDeviceDynamicManagement.getElectricity());
        list.add(baseCameraManagement.getMeasurePointName());
        if (list1.size() <= 0) {
            list.add(1);
        } else {
            MsgAlarm msgAlarm = list1.get(0);
            list.add(msgAlarm.getIsLock());
        }
        return AjaxResult.success(list);
    }

    /**
     * 台账管理导入
     *
     * @param file
     * @param updateSupport
     * @return
     * @throws Exception
     */
    @PostMapping("/importBaseAccountManagement")
    @Log(title = "相机台账管理", businessType = BusinessType.IMPORT)
    public AjaxResult importBaseAccountManagement(MultipartFile file, boolean updateSupport) throws Exception {
        try {
            List<BaseCameraManagementVos> accountManageList;
            ExcelUtil<BaseCameraManagementVos> util = new ExcelUtil<BaseCameraManagementVos>(BaseCameraManagementVos.class);
            ExcelUtil<String> utils = new ExcelUtil<String>(String.class);
            InputStream resourceAsStream = this.getClass().getResourceAsStream("/template/baseCameraManagement.xlsx");
            List<String> accountManage = utils.importExcels(StringUtils.EMPTY, file.getInputStream(), 0);
            List<String> accountManageLists = utils.importExcels(StringUtils.EMPTY, resourceAsStream, 0);
            boolean stats = accountManage.stream().sorted().collect(Collectors.joining())
                    .equals(accountManageLists.stream().sorted().collect(Collectors.joining()));
            if (stats) {
                accountManageList = util.importExcel(file.getInputStream());
            } else {
                return AjaxResult.error("表头格式不正确,请按照模板样式编写");
            }
            String message = baseCameraManagementService.importBaseAccountManagement(accountManageList, updateSupport, getUserId());
            return AjaxResult.success(message);
        } catch (Exception exception) {
            return AjaxResult.error(exception.getMessage());
        }
    }

    /**
     * 导出台账管理
     *
     * @param response
     */
    @PostMapping("/exportBaseAccountManagement")
    @Log(title = "相机台账管理", businessType = BusinessType.EXPORT)
    public void exportBaseAccountManagement(HttpServletResponse response, @RequestBody BaseCameraManagementVo baseCameraManagementVo) {
        baseCameraManagementVo = (BaseCameraManagementVo) dataScoreUtil.setDataScore(getUserId(), baseCameraManagementVo);
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getBeginMile())) {
            baseCameraManagementVo.setBeginMile(baseCameraManagementVo.getBeginMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getEndMile())) {
            baseCameraManagementVo.setEndMile(baseCameraManagementVo.getEndMile() * 1000);
        }
        List<BaseCameraManagement> baseCameraManagementList = baseCameraManagementService.baseCameraManagementList(baseCameraManagementVo);
        for (BaseCameraManagement baseCameraManagement : baseCameraManagementList) {
            SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
            baseCameraManagement.setDeptName(sysDept.getDeptName());
            LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                queryWrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
            baseCameraManagement.setRailwayName(baseRailwayManagement.getRailwayName());
            baseCameraManagement.setRailwayType(baseRailwayManagement.getRailwayType());
            String mils = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            baseCameraManagement.setInstallMiles(mils);
            if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile())) {
                String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
                baseCameraManagement.setEndMiles(end);
            }
            if (!ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
                String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
                baseCameraManagement.setBeginMiles(begin);
            }
        }
        baseCameraManagementList.sort(new Comparator<BaseCameraManagement>() {
            @Override
            public int compare(BaseCameraManagement o1, BaseCameraManagement o2) {
                String name1 = ChineseToPingyin.convertHanziToPinyin(o1.getRailwayName());
                String substring1 = name1.substring(0, 1);
                String name2 = ChineseToPingyin.convertHanziToPinyin(o2.getRailwayName());
                String substring2 = name2.substring(0, 1);
                int i = substring1.compareTo(substring2);
                if (i == 0) {
                    return o1.getInstallMile() - o2.getInstallMile();
                }
                return i;
            }
        });
        ExcelUtil<BaseCameraManagement> util = new ExcelUtil<>(BaseCameraManagement.class);
        util.exportExcel(response, baseCameraManagementList, "台账管理相机数据");
    }


    @GetMapping("/downloaExcel")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public void downloadZip(HttpServletResponse response) {
        try {
            InputStream resourceAsStream = this.getClass().getResourceAsStream("/template/baseCameraManagement.xlsx");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, "相机台账管理信息模板.xlsx");
            FileUtils.writeBytesByInput(resourceAsStream, response.getOutputStream());
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    @GetMapping("/api/records/{cameraCode}")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult apiRecords(@PathVariable @NotNull(message = "相机编码不能为空") String cameraCode) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + cameraCode + "】不存在");
        }
        return new AjaxResult(200, "ok", CameraUtil.getPlayFlv(cameraCode, one.getChannel()));
    }

    @GetMapping("/api/records/{cameraCode}/{flay}")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult apiRecords(@PathVariable @NotNull(message = "相机编码不能为空") String cameraCode,
                                 @PathVariable @NotNull(message = "条件不能为空") boolean flay) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + cameraCode + "】不存在");
        }
        return new AjaxResult(200, "ok", CameraUtil.getPlayFlv(cameraCode, one.getChannel(), flay));
    }

    @GetMapping("/api/records/{cameraCode}/{flay}/{format}")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult apiRecords(@PathVariable @NotNull(message = "相机编码不能为空") String cameraCode,
                                 @PathVariable @NotNull(message = "条件不能为空") boolean flay,
                                 @PathVariable @NotNull(message = "格式不能为空") String format) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + cameraCode + "】不存在");
        }
        return new AjaxResult(200, "ok", CameraUtil.getPlayFlv(cameraCode, one.getChannel(), flay, format));
    }

    @GetMapping("/api/recordsList")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult apiRecordsList(@RequestBody
                                     @NotNull(message = "相机编码不能为空")
                                     List<String> cameraCodes) {
        List<String> list = new ArrayList<>();
        for (String cameraCode : cameraCodes) {
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
            BaseCameraManagement one = baseCameraManagementService.getOne(lw);
            if (ObjectUtils.isEmpty(one)) {
                throw new BaseException("相机编号【" + cameraCode + "】不存在");
            }
            String playFlv = null;
            if (list.size() > 5) {
                playFlv = CameraUtil.getPlayFlv(cameraCode, one.getChannel(), false);
            } else {
                playFlv = CameraUtil.getPlayFlv(cameraCode, one.getChannel());
            }
            list.add(playFlv);
        }

        return new AjaxResult(200, "ok", list);
    }


    @PostMapping("/api/invite")
    public AjaxResult apiInvite(@Validated @RequestBody BaseCameraResVo vo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, vo.getCameraCode());
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + vo.getCameraCode() + "】不存在");
        }
        // CameraUtil.historyPlayList(one.getChannel(), vo.getStartTime(), vo.getEntTime(), caneraConfig.getMappingUrl())
        return success();
    }

    @PostMapping("/api/play")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult play(@Validated @RequestBody BaseCameraResVo vo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, vo.getCameraCode());
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + vo.getCameraCode() + "】不存在");
        }
        String path = CameraUtil.historyPlayListStr(one.getChannel(), vo.getStartTime(), vo.getEntTime(), false);
        if (org.apache.commons.lang3.StringUtils.isBlank(path)) {
            throw new BaseException("当前相机无视频录像");
        }
        return new AjaxResult(200, "ok", serverConfig.getUrl() + path);
//        return success(CameraUtil.historyPlayList(one.getChannel(), vo.getStartTime(), vo.getEntTime()));
    }

    @GetMapping("/totalNumberOfDevices")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult totalNumberOfDevices() {
        long count = baseCameraManagementService.count();
        return success(count);
    }

    /**
     * 视频服务树形结构
     *
     * @param
     * @param
     * @param
     * @return
     */
    @PostMapping("/cameraTree")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult cameraTree(@RequestBody CameraTreeVo cameraTreeVo) {
        cameraTreeVo = (CameraTreeVo) dataScoreUtil.setDataScore(getUserId(), cameraTreeVo);
        int count = 0;
        if (!ObjectUtils.isEmpty(cameraTreeVo.getMils())) {
            cameraTreeVo.setMils(cameraTreeVo.getMils() * 1000);
        }
        CameraTree cameraTree = baseDeviceDynamicManagementService.cameraTree(cameraTreeVo);
        List<CameraTree> children = cameraTree.getChildren();
        children.sort(new Comparator<CameraTree>() {
            @Override
            public int compare(CameraTree o1, CameraTree o2) {
                if (ObjectUtils.isEmpty(o1.getNumber())&& ObjectUtils.isEmpty(o2.getNumber())) {
                    int mileageDiff = o1.getNumber() - o2.getNumber();
                    if (mileageDiff == 0) {
                        String name1 = ChineseToPingyin.convertHanziToPinyin(o1.getRailwayName());
                        String substring1 = name1.substring(0, 1);
                        String name2 = ChineseToPingyin.convertHanziToPinyin(o2.getRailwayName());
                        String substring2 = name2.substring(0, 1);
                        return substring1.compareTo(substring2);
                    } else {
                        return mileageDiff;
                    }
                }else {
                    String name1 = ChineseToPingyin.convertHanziToPinyin(o1.getRailwayName());
                    String substring1 = name1.substring(0, 1);
                    String name2 = ChineseToPingyin.convertHanziToPinyin(o2.getRailwayName());
                    String substring2 = name2.substring(0, 1);
                    return substring1.compareTo(substring2);
                }
            }
        });
        for (CameraTree child : children) {
            List<CameraTree> children1 = child.getChildren();
            children1.sort(new Comparator<CameraTree>() {
                @Override
                public int compare(CameraTree o1, CameraTree o2) {
                    String name1 = ChineseToPingyin.convertHanziToPinyin(o1.getPublicWorksSection());
                    String substring1 = name1.substring(0, 1);
                    String name2 = ChineseToPingyin.convertHanziToPinyin(o2.getPublicWorksSection());
                    String substring2 = name2.substring(0, 1);
                    return substring1.compareTo(substring2);
                }
            });
            for (CameraTree tree3 : children1) {
                List<CameraTree> cameraTreeList = tree3.getChildren();
                cameraTreeList.sort(new Comparator<CameraTree>() {
                    @Override
                    public int compare(CameraTree o1, CameraTree o2) {
                        int mileageDiff = o1.getMileage() - o2.getMileage();
                        // 如果里程相同，则比较方向
                        if (mileageDiff == 0) {
                            String[] split1 = o1.getName().split("-");
                            String[] split2 = o2.getName().split("-");

                            // 根据方向给予优先级
                            int priority1 = getDirectionPriority(split1[1]);
                            int priority2 = getDirectionPriority(split2[1]);

                            return Integer.compare(priority1, priority2);
                        } else {
                            return mileageDiff;
                        }
                    }
                });
                for (CameraTree tree : cameraTreeList) {
                    List<CameraTree> children2 = tree.getChildren();
                    if (children2.size() < 2) {
                        String code = children2.get(0).getCode();
                        tree.setCode(code);
                        children2.remove(0);
                    } else {
                        for (CameraTree cameraTree1 : children2) {
                            count = count + 1;
                            cameraTree1.setName("相机" + count);
                        }
                        count = 0;
                    }
                }
            }
        }
        return success(cameraTree);
    }

    private int getDirectionPriority(String direction) {
        switch (direction) {
            case "上行":
                return 1; // 高优先级
            case "下行":
                return 2; // 次优先级
            case "上下行":
                return 3; // 低优先级
            default:
                return Integer.MAX_VALUE; // 未知方向，最低优先级
        }
    }

    @GetMapping("/endRecording/{id}")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult endRecording(@PathVariable Long id) {
        List<SysDictData> data = dictTypeService.selectDictDataByType("management_transcribe_switch");
        if (data.get(0).getDictValue().equals("true")) {
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
            if (ObjectUtils.isEmpty(baseCameraManagement)) {
                return error("查不到该ID的相机台账数据!!!");
            }
            log.info("endRecording------------" + baseCameraManagement.getTaskId());
            String start = HttpUtils.sendGet(CameraUtil.endRecording(baseCameraManagement.getCameraCode(), baseCameraManagement.getChannel()));
            log.info("endRecording------------" + start);
            if ("success".equals(start)) {
                baseCameraManagement.setStartRecording(2);
                baseCameraManagement.setTaskId("");
                baseCameraManagement.setUpdateBy(getUserId());
            } else {
                baseCameraManagement.setStartRecording(2);
                baseCameraManagement.setTaskId("");
                baseCameraManagement.setUpdateBy(getUserId());
                log.error("关闭视频录制异常日志如下:" + start);
            }
            if (baseCameraManagementService.updateById(baseCameraManagement)) {
                return success();
            }
            return error();
        } else {
            return error("已自动开启录制,关闭按钮已失效");
        }
    }


    @GetMapping("/startRecording/{id}")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult startRecording(@PathVariable Long id) throws Exception {
        List<SysDictData> data = dictTypeService.selectDictDataByType("management_transcribe_switch");
        if (data.get(0).getDictValue().equals("true")) {
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
            log.info("startRecording------CameraCode------" + baseCameraManagement.getCameraCode());
            log.info("startRecording------Channel------" + baseCameraManagement.getChannel());
            String start = HttpClientUtil.gets(CameraUtil.startRecording(baseCameraManagement.getCameraCode(), baseCameraManagement.getChannel()));
            log.info("startRecording------------" + start);
            if (start.startsWith(baseCameraManagement.getCameraCode() + "/" + baseCameraManagement.getChannel())) {
                log.info("startRecording 返回成功！！1");
                baseCameraManagement.setStartRecording(1);
                baseCameraManagement.setTaskId(start);
                baseCameraManagement.setUpdateBy(getUserId());
            } else if (start.startsWith("recorder")) {
                baseCameraManagement.setStartRecording(1);
                baseCameraManagement.setTaskId(baseCameraManagement.getCameraCode() + "/" + baseCameraManagement.getChannel() + "/flv");
                baseCameraManagementService.updateById(baseCameraManagement);
            } else if (start.startsWith("Stream")) {
                return error("开启视频录制失败:流不存在");
            }
            if (baseCameraManagementService.updateById(baseCameraManagement)) {
                return success();
            } else {
                return error("开启视频录制失败");
            }
        } else {
            return error("已自动开启录制,无需按钮再次开启录制");
        }
    }

    @GetMapping(value = "/getAlarm/{cameraCode}")
    @Log(title = "相机台账管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getAlarm(@PathVariable String cameraCode) {
        LambdaQueryWrapper<BaseCameraManagement> queryWrapper = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(cameraCode)) {
            queryWrapper.eq(BaseCameraManagement::getCameraCode, cameraCode);
        }
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(queryWrapper);
        if (!ObjectUtils.isEmpty(baseCameraManagement)) {
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            LambdaQueryWrapper<MsgAlarm> wrapper1 = new LambdaQueryWrapper<MsgAlarm>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
                wrapper1.eq(MsgAlarm::getCameraCode, baseCameraManagement.getCameraCode());
                wrapper1.orderByDesc(MsgAlarm::getAlarmTime);
            }
            List<MsgAlarm> list = msgAlarmService.list(wrapper1);
            if (list.size() > 0) {
                msgAlarmResp.setFailureToAlarm(1);
                MsgAlarm msgAlarm = list.get(0);
                if (StringUtils.isNotEmpty(msgAlarm.getReleasedBy())) {
                    BaseUser user = baseUserService.getUser(msgAlarm.getReleasedBy());
                    msgAlarm.setReleasedByName(user.getNickName());
                }
                msgAlarmResp.setAlarmType(msgAlarm.getAlarmType());
                msgAlarmResp.setIsLock(msgAlarm.getIsLock());
                msgAlarmResp.setReleasedTime(msgAlarm.getReleasedTime());
                msgAlarmResp.setAlarmTime(msgAlarm.getAlarmTime());
                msgAlarmResp.setReleasedType(msgAlarm.getReleasedType());
                msgAlarmResp.setReleasedReason(msgAlarm.getReleasedReason());
                msgAlarmResp.setAlarmAttr(msgAlarm.getAlarmAttr());
                msgAlarmResp.setContent(msgAlarm.getContent());
                msgAlarmResp.setReleasedByName(msgAlarm.getReleasedByName());
                msgAlarmResp.setReleasedReason(msgAlarm.getReleasedReason());
                String[] split1 = msgAlarm.getAlarmAttr().split(",");
                String stringBuilder = "画面大小为" + split1[0] + "×" + split1[1] + "," +
                        "泥石流范围为左上角坐标(" + split1[2] + "," + split1[3] + ")" +
                        "大小" + split1[4] + "×" + split1[5];
                msgAlarmResp.setAlarmAttrName(stringBuilder);
                if (!ObjectUtils.isEmpty(msgAlarmResp.getReleasedType())) {
                    msgAlarmResp.setReleasedTypeName(msgAlarmResp.getReleasedType() == 1 ? "实报" : "误报");
                }
                if (!StringUtils.isEmpty(msgAlarm.getImageUrl())) {
                    String[] split = msgAlarm.getImageUrl().split(";");
                    ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
                    log.info("publish--------" + objects);
                    msgAlarmResp.setImageUrls(objects);
                }
            } else {
                msgAlarmResp.setFailureToAlarm(2);
            }
            BeanUtils.copyProperties(baseCameraManagement, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile()) && !ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
                String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
                String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
                String endMile = begin + "~" + end;
                msgAlarmResp.setBeginMile(endMile);
            }
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            if (!ObjectUtils.isEmpty(baseCameraManagement.getInstallLongitude()) && !ObjectUtils.isEmpty(baseCameraManagement.getInstallLatitude())) {
                msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            } else {
                msgAlarmResp.setInstallLongitude(null);
            }
            msgAlarmResp.setLineDir(baseCameraManagement.getLineDir());
            SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
            msgAlarmResp.setDeptName(sysDept.getDeptName());
            msgAlarmResp.setCardNum(baseCameraManagement.getCardNum());
            msgAlarmResp.setCameraSn(baseCameraManagement.getCameraSn());
            return AjaxResult.success(msgAlarmResp);
        } else {
            return error("该相机编码在相机台账表中不存在");
        }
    }


    @PostMapping("/api/record")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult record(@Validated @RequestBody BaseCameraResVo vo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, vo.getCameraCode());
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + vo.getCameraCode() + "】不存在");
        }
        String recordUrl = CameraUtil.getRecordUrl(one.getCameraCode(), one.getChannel(), vo.getStartTime(), vo.getEntTime());
        if (recordUrl.contains("当前相机暂无回放视频")) {
            return AjaxResult.error(recordUrl);
        } else {
            return AjaxResult.success("ok", recordUrl);
        }
    }

    @GetMapping("/api/getRecordByAlarmId")
    @Log(title = "相机台账管理", businessType = BusinessType.OTHER)
    public AjaxResult getRecordByAlarmId(@RequestParam("alarmId") String alarmId) {

        LambdaQueryWrapper<MsgAlarm> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(MsgAlarm::getAlarmId, alarmId);
        MsgAlarm msgAlarm = msgAlarmService.getOne(wrapper);
        if (ObjectUtils.isEmpty(msgAlarm.getReleasedUrl())) {
            return error("当前相机暂无回放视频");
        } else {
            return new AjaxResult(200, "ok", caneraConfig.getRecordUrl() + msgAlarm.getReleasedUrl());
        }
    }

    /**
     * 测试minio删除图片
     *
     * @return
     */
    @GetMapping("/removePrice/{number}")
    public AjaxResult removePrice(@PathVariable Integer number) {
        log.info("removePrice");
        cameraUtil.test(number);
        return success();
    }

    /**
     * 夜视
     *
     * @param deviceStateResp
     * @return
     */
    @GetMapping("/nightVision/{code}/{mode}")
    public AjaxResult nightVision(@PathVariable String code, Integer mode) {
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(new LambdaQueryWrapper<BaseCameraManagement>()
                .eq(BaseCameraManagement::getCameraCode, code));
        PublishClient.configFrameRate(baseCameraManagement, mode);
        return success();
    }

    /**
     * 切换帧率
     *
     * @param code
     * @return
     */
    @GetMapping("/switchFrameRate")
    public AjaxResult switchFrameRate(@RequestParam("code") String code, @RequestParam("fps") String fps) {
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(new LambdaQueryWrapper<BaseCameraManagement>()
                .eq(BaseCameraManagement::getCameraCode, code));
        PublishClient.confidenceCoefficient(baseCameraManagement, fps);
        return success();
    }

    /**
     * 设备状态页面
     *
     * @param deviceStateResp
     * @return
     */
    @PostMapping("/deviceState/page")
    @Log(title = "相机台账管理", businessType = BusinessType.SELECT)
    public AjaxResult deviceStatePage(@RequestBody DeviceStateResp deviceStateResp) {
        deviceStateResp = (DeviceStateResp) dataScoreUtil.setDataScore(getUserId(), deviceStateResp);
        IPage<DeviceStateResp> page = baseCameraManagementService.deviceStatePage(deviceStateResp);
        page.setTotal(page.getTotal());
        page.setCurrent(page.getCurrent());
        page.setPages(page.getPages());
        if (!ObjectUtils.isEmpty(page) && page.getRecords().size() > 0) {
            List<DeviceStateResp> dto1 = page.getRecords().stream().map(o -> {
                String mils = AppendUtils.stringAppend(o.getInstallMile());
                o.setInstallMileName(mils);
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }


    /**
     * 导出设备状态页面
     *
     * @param response
     */
    @PostMapping("/exportDeviceState")
    @Log(title = "相机台账管理", businessType = BusinessType.EXPORT)
    public void exportDeviceState(HttpServletResponse response, @RequestBody DeviceStateResp deviceStateResp) {
        deviceStateResp = (DeviceStateResp) dataScoreUtil.setDataScore(getUserId(), deviceStateResp);
        List<DeviceStateResp> deviceStateRespList = baseCameraManagementService.deviceStateRespList(deviceStateResp);
        for (DeviceStateResp deviceStateResp1 : deviceStateRespList) {
            String mils = AppendUtils.stringAppend(deviceStateResp1.getInstallMile());
            deviceStateResp1.setInstallMileName(mils);
        }
        ExcelUtil<DeviceStateResp> util = new ExcelUtil<>(DeviceStateResp.class);
        util.exportExcel(response, deviceStateRespList, "台账设备状态数据失败");
    }

    /**
     * 获取离线相机和电量低的相机
     */
    @GetMapping("/getMalfunctionAndLowCameraManagement")
    public AjaxResult getMalfunctionAndLowCameraManagement() {
        HashMap<String, Integer> map = new HashMap<>();
        DeviceStateResp deviceStateResp1 = new DeviceStateResp();
        DeviceStateResp deviceStateResp = (DeviceStateResp) dataScoreUtil.setDataScore(getUserId(),deviceStateResp1);
        List<DeviceStateResp> deviceStateRespList = baseCameraManagementService.deviceStateRespList(deviceStateResp);
        List<DeviceStateResp> isDisable = deviceStateRespList.stream().filter(f -> f.getIsDisable().equals("2")).collect(Collectors.toList());
        map.put("isDisable", isDisable.size());
        List<DeviceStateResp> electricity = deviceStateRespList.stream().filter(f -> f.getElectricity()<=10).collect(Collectors.toList());
        map.put("electricity", electricity.size());
        map.put("count", isDisable.size() + electricity.size());
        map.put("readCount", isDisable.size()+electricity.size());
        return AjaxResult.success(map);
    }

    /**
     * 修改设备状态信息(1已读2未读)
     * 废弃
     */
//    @PostMapping("/updateState")
//    public AjaxResult updateState(@RequestBody List<DeviceStateResp> deviceStateRespList) {
//        List<String> list = deviceStateRespList.stream().map(DeviceStateResp::getCameraCode).collect(Collectors.toList());
//        BaseDeviceDynamicManagement dynamicManagement = new BaseDeviceDynamicManagement();
//        LambdaQueryWrapper<BaseDeviceDynamicManagement> wrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
//        wrapper.in(BaseDeviceDynamicManagement::getCameraCode, list);
//        boolean update = baseDeviceDynamicManagementService.update(dynamicManagement, wrapper);
//        if (update) {
//            return AjaxResult.success();
//        } else {
//            return AjaxResult.error();
//        }
//    }




    /**
     * 首页设备状态
     * @return
     */
    @GetMapping("cameraStateTotal")
    public AjaxResult cameraStateTotal() {
     return baseCameraManagementService.cameraStateTotal();
    }

    /**
     * 首页设备状态列表
     * @return
     */
    @GetMapping("cameraStateList")
    public AjaxResult cameraStateList() {
        return baseCameraManagementService.cameraStateList();
    }


    /**
     * 批量修改相机已读未读状态
     * 废弃
     * @return
     */
//    @GetMapping("updateStateAll/{state}")
//    public AjaxResult updateStateAll(@PathVariable Integer state) {
//        return baseCameraManagementService.updateStateAll(state);
//    }
}

