package com.ozs.web.controller.accountmanagment;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.annotation.Log;
import com.ozs.common.constant.Constants;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.core.domain.entity.SysDictType;
import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.common.core.redis.RedisCache;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.file.FileUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.service.entity.*;
import com.ozs.service.entity.vo.*;
import com.ozs.service.mapper.MsgAlarmMapper;
import com.ozs.service.service.*;
import com.ozs.system.service.DataScoreUtil;
import com.ozs.system.service.ISysDeptService;
import com.ozs.system.service.ISysDictDataService;
import com.ozs.web.core.util.CameraUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 报警信息表 前端控制器
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@RestController
@RequestMapping("/service/msgAlarm")
public class MsgAlarmController extends BaseController {
    private static final Logger log = LoggerFactory.getLogger(MsgAlarmController.class);
    @Resource
    MsgAlarmService msgAlarmService;
    @Resource
    MsgAlarmMapper msgAlarmMapper;
    @Resource
    MsgAlarmFrequencyService msgAlarmFrequencyService;
    @Autowired
    BaseCameraManagementService baseCameraManagementService;
    @Autowired
    BaseRailwayManagementService baseRailwayManagementService;
    @Autowired
    private MsgWebPushService msgWebPushService;
    @Autowired
    private ISysDictDataService dictDataService;
    @Autowired
    private DataScoreUtil dataScoreUtil;
    @Resource
    private BaseUserService baseUserService;
    @Autowired
    private ISysDeptService deptService;
    @Autowired
    private MsgAppPushService msgAppPushService;
    @Autowired
    private RedisService redisService;


    /**
     * web更多消息-总数，已读数
     */
    @PostMapping("/messageMoreNum")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult messageMoreNum(@RequestBody MsgAlarmVo msgAlarmVo) {
        // MsgAlarmVo msgAlarmVo1 = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        Map<String, Integer> messageMore = msgWebPushService.messageMoreNum(getUserId());
        return AjaxResult.success(messageMore);
    }

    /**
     * web更多消息
     */
    @PostMapping("/messageMore")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult messageMore(@RequestBody MsgAlarmVo msgAlarmVo) {
        // MsgAlarmVo msgAlarmVo1 = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        String userId = getUserId();
        if (!StringUtils.isEmpty(userId)) {
            msgAlarmVo.setUserId(userId);
        }
        IPage<MsgAlarm> messageMore = msgWebPushService.messageMore(msgAlarmVo);
        return AjaxResult.success(messageMore);
    }

    /**
     * web报警未读消息个数+未读消息列表
     */
    @GetMapping("/webAlarm")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult webAlarmNum() {
        // MsgAlarmVo msgAlarm = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), new MsgAlarmVo());
        Map webAlarm = msgWebPushService.webAlarm(getUserId());
        return AjaxResult.success(webAlarm);
    }

    /**
     * web消息全部已读
     */
    @PostMapping("/readAll")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult readAll() {
        return toAjax(msgWebPushService.readAll(getUserId()));
    }

    /**
     * web消息已读
     */
    @PostMapping("/read")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult read(@RequestBody MsgWebPush msgWebPush) {
//        if (StringUtils.isEmpty(msgWebPush) || ObjectUtils.isEmpty(msgWebPush.getPushId())) {
//            return AjaxResult.error("PushId不能为空！");
//        }
        return toAjax(msgWebPushService.read(msgWebPush));
    }

    /**
     * 首页报警信息列表
     * 首页左侧报警默认为全部，先按照未解除、已解除排序，然后再按照时间倒序排序
     *
     * @param msgAlarmVo
     * @return
     */
    @PostMapping("/listHome")
    @Log(title = "首页", businessType = BusinessType.SELECT)
    public AjaxResult selectMsgAlarmHome(@RequestBody MsgAlarmVo msgAlarmVo) {
        msgAlarmVo = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        if (!ObjectUtils.isEmpty(msgAlarmVo.getAlarmMile())) {
            msgAlarmVo.setAlarmMile(msgAlarmVo.getAlarmMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getBeginMile())) {
            msgAlarmVo.setBeginMile(msgAlarmVo.getBeginMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getEndMile())) {
            msgAlarmVo.setEndMile(msgAlarmVo.getEndMile() * 1000);
        }
        IPage<MsgAlarm> page = msgAlarmService.queryPageHome(msgAlarmVo);
        page.setTotal(page.getTotal());
        page.setCurrent(page.getCurrent());
        page.setPages(page.getPages());
        if (!ObjectUtils.isEmpty(page) && page.getRecords().size() > 0) {
            List<MsgAlarm> dto1 = page.getRecords().stream().map(o -> {
                String[] split = o.getImageUrl().split(";");
                ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
                o.setImageUrls(objects);
                LambdaQueryWrapper<BaseCameraManagement> cameraManagementLambdaQueryWrapper = new LambdaQueryWrapper<BaseCameraManagement>();
                if (!ObjectUtils.isEmpty(o.getCameraCode())) {
                    cameraManagementLambdaQueryWrapper.eq(BaseCameraManagement::getCameraCode, o.getCameraCode());
                }
                BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(cameraManagementLambdaQueryWrapper);
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(cameraManagement.getRailwayCode())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, cameraManagement.getRailwayCode());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
                o.setRailwayName(baseRailwayManagement.getRailwayName());
                if (StringUtils.isNotEmpty(o.getReleasedBy())) {
                    BaseUser user = baseUserService.getUser(o.getReleasedBy());
                    o.setReleasedByName(user.getNickName());
                }
                String mils = AppendUtils.stringAppend(cameraManagement.getInstallMile());
                o.setAlarmMiles(mils);
                if (!ObjectUtils.isEmpty(cameraManagement.getEndMile())) {
                    String end = AppendUtils.stringAppend(cameraManagement.getEndMile());
                    o.setEndMile(end);
                }
                if (!ObjectUtils.isEmpty(cameraManagement.getBeginMile())) {
                    String begin = AppendUtils.stringAppend(cameraManagement.getBeginMile());
                    o.setBeginMile(begin);
                }
                LambdaQueryWrapper<MsgAlarmFrequency> wrapper = new LambdaQueryWrapper<MsgAlarmFrequency>();
                if (!ObjectUtils.isEmpty(o.getAlarmId())) {
                    wrapper.eq(MsgAlarmFrequency::getAlarmId, o.getAlarmId());
                }
                o.setAlarmCount((int) msgAlarmFrequencyService.count(wrapper));
                if (!ObjectUtils.isEmpty(o.getReleasedType())) {
                    o.setReleasedTypeName(o.getReleasedType() == 1 ? "实报" : "误报");
                }
                String[] split1 = o.getAlarmAttr().split(",");
                String stringBuilder = "画面,大小为" + split1[0] + "×" + split1[1] + "," +
                        "泥石流范围为左上角坐标(" + split1[2] + "," + split1[3] + ")" +
                        "大小" + split1[4] + "×" + split1[5];
                o.setAlarmAttr(stringBuilder);
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }

    /**
     * 报警信息分页
     *
     * @param msgAlarmVo
     * @return
     */
    @PostMapping("/list")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult selectMsgAlarm(@RequestBody MsgAlarmVo msgAlarmVo) {
        msgAlarmVo = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        if (!ObjectUtils.isEmpty(msgAlarmVo.getAlarmMile())) {
            msgAlarmVo.setAlarmMile(msgAlarmVo.getAlarmMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getBeginMile())) {
            msgAlarmVo.setBeginMile(msgAlarmVo.getBeginMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getEndMile())) {
            msgAlarmVo.setEndMile(msgAlarmVo.getEndMile() * 1000);
        }
        IPage<MsgAlarm> page = msgAlarmService.queryPage(msgAlarmVo);
        page.setTotal(page.getTotal());
        page.setCurrent(page.getCurrent());
        page.setPages(page.getPages());

        if (!ObjectUtils.isEmpty(page) && page.getRecords().size() > 0) {
            List<MsgAlarm> dto1 = page.getRecords().stream().map(o -> {
                String[] split = o.getImageUrl().split(";");
                ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
                o.setImageUrls(objects);
                LambdaQueryWrapper<BaseCameraManagement> cameraManagementLambdaQueryWrapper = new LambdaQueryWrapper<BaseCameraManagement>();
                if (!ObjectUtils.isEmpty(o.getCameraCode())) {
                    cameraManagementLambdaQueryWrapper.eq(BaseCameraManagement::getCameraCode, o.getCameraCode());
                }
                BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(cameraManagementLambdaQueryWrapper);
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(cameraManagement.getRailwayCode())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, cameraManagement.getRailwayCode());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
                o.setRailwayName(baseRailwayManagement.getRailwayName());
                if (StringUtils.isNotEmpty(o.getReleasedBy())) {
                    BaseUser user = baseUserService.getUser(o.getReleasedBy());
                    o.setReleasedByName(user.getNickName());
                }
                String mils = AppendUtils.stringAppend(cameraManagement.getInstallMile());
                o.setAlarmMiles(mils);
                if (!ObjectUtils.isEmpty(cameraManagement.getEndMile())) {
                    String end = AppendUtils.stringAppend(cameraManagement.getEndMile());
                    o.setEndMile(end);
                }
                if (!ObjectUtils.isEmpty(cameraManagement.getBeginMile())) {
                    String begin = AppendUtils.stringAppend(cameraManagement.getBeginMile());
                    o.setBeginMile(begin);
                }
                LambdaQueryWrapper<MsgAlarmFrequency> wrapper = new LambdaQueryWrapper<MsgAlarmFrequency>();
                if (!ObjectUtils.isEmpty(o.getAlarmId())) {
                    wrapper.eq(MsgAlarmFrequency::getAlarmId, o.getAlarmId());
                }
                o.setAlarmCount((int) msgAlarmFrequencyService.count(wrapper));
                if (!ObjectUtils.isEmpty(o.getReleasedType())) {
                    o.setReleasedTypeName(o.getReleasedType() == 1 ? "实报" : "误报");
                }
                String[] split1 = o.getAlarmAttr().split(",");
                String stringBuilder = "画面,大小为" + split1[0] + "×" + split1[1] + "," +
                        "泥石流范围为左上角坐标(" + split1[2] + "," + split1[3] + ")" +
                        "大小" + split1[4] + "×" + split1[5];
                o.setAlarmAttr(stringBuilder);
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }

    /**
     * 导出报警消息
     *
     * @param response
     */
    @PostMapping("/exportMsgAlarm")
    @Log(title = "报警信息管理", businessType = BusinessType.EXPORT)
    public void exportMsgAlarm(HttpServletResponse response, @RequestBody MsgAlarmVo msgAlarmVo) {
        msgAlarmVo = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        if (!ObjectUtils.isEmpty(msgAlarmVo.getAlarmMile())) {
            msgAlarmVo.setAlarmMile(msgAlarmVo.getAlarmMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getBeginMile())) {
            msgAlarmVo.setBeginMile(msgAlarmVo.getBeginMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getEndMile())) {
            msgAlarmVo.setEndMile(msgAlarmVo.getEndMile() * 1000);
        }
        List<MsgAlarm> msgAlarmList = msgAlarmService.msgAlarmList(msgAlarmVo);
        for (MsgAlarm msgAlarm : msgAlarmList) {
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarm.setImageUrls(objects);
            LambdaQueryWrapper<BaseCameraManagement> cameraManagementLambdaQueryWrapper = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                cameraManagementLambdaQueryWrapper.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(cameraManagementLambdaQueryWrapper);
            LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(cameraManagement.getRailwayCode())) {
                queryWrapper.eq(BaseRailwayManagement::getRailwayCode, cameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
            msgAlarm.setRailwayName(baseRailwayManagement.getRailwayName());
            if (StringUtils.isNotEmpty(msgAlarm.getReleasedBy())) {
                BaseUser user = baseUserService.getUser(msgAlarm.getReleasedBy());
                msgAlarm.setReleasedByName(user.getNickName());
            }
            String mils = AppendUtils.stringAppend(cameraManagement.getInstallMile());
            msgAlarm.setAlarmMiles(mils);
            if (!ObjectUtils.isEmpty(cameraManagement.getEndMile())) {
                String end = AppendUtils.stringAppend(cameraManagement.getEndMile());
                msgAlarm.setEndMile(end);
            }
            if (!ObjectUtils.isEmpty(cameraManagement.getBeginMile())) {
                String begin = AppendUtils.stringAppend(cameraManagement.getBeginMile());
                msgAlarm.setBeginMile(begin);
            }
            LambdaQueryWrapper<MsgAlarmFrequency> wrapper = new LambdaQueryWrapper<MsgAlarmFrequency>();
            if (!ObjectUtils.isEmpty(msgAlarm.getAlarmId())) {
                wrapper.eq(MsgAlarmFrequency::getAlarmId, msgAlarm.getAlarmId());
            }
            msgAlarm.setAlarmCount((int) msgAlarmFrequencyService.count(wrapper));
            if (!ObjectUtils.isEmpty(msgAlarm.getReleasedType())) {
                msgAlarm.setReleasedTypeName(msgAlarm.getReleasedType() == 1 ? "实报" : "误报");
            }
            if (msgAlarm.getLineDir() == 1) {
                msgAlarm.setLineDirName("上行");
            } else if (msgAlarm.getLineDir() == 2) {
                msgAlarm.setLineDirName( "下行");
            } else if (msgAlarm.getLineDir() == 3) {
                msgAlarm.setLineDirName( "上下行");
            }


            msgAlarm.setIsLockName(msgAlarm.getIsLock() == 1 ? "已解除" : "未解除");
            LambdaQueryWrapper<SysDictData> dict = new LambdaQueryWrapper<>();
            dict.eq(SysDictData::getDictType, "sys_alarm_type");
            dict.eq(SysDictData::getDictValue, msgAlarm.getAlarmType());
            SysDictData one = dictDataService.getOne(dict);
            msgAlarm.setAlarmTypeName(one.getDictLabel());
            String[] split1 = msgAlarm.getAlarmAttr().split(",");
            String stringBuilder = "画面,大小为" + split1[0] + "×" + split1[1] + "," +
                    "泥石流范围为左上角坐标(" + split1[2] + "," + split1[3] + ")" +
                    "大小" + split1[4] + "×" + split1[5];
            msgAlarm.setAlarmAttr(stringBuilder);
        }
        ExcelUtil<MsgAlarm> util = new ExcelUtil<>(MsgAlarm.class);
        util.exportExcel(response, msgAlarmList, "报警信息管理");
    }

    @GetMapping(value = "/getByAlarmId/{alarmId}")
    @Log(title = "报警信息管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getByAlarmId(@PathVariable String alarmId) {
        MsgAlarm msgAlarm = msgAlarmService.getByAlarmId(alarmId);
        if (!ObjectUtils.isEmpty(msgAlarm)) {
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarm.setImageUrls(objects);
            if (StringUtils.isNotEmpty(msgAlarm.getReleasedBy())) {
                BaseUser user = baseUserService.getUser(msgAlarm.getReleasedBy());
                msgAlarm.setReleasedByName(user.getNickName());
            }
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile())&&!ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
                String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
                String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
                String endMile = begin + "-" + end;
                msgAlarmResp.setBeginMile(endMile);
            }
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            if (!ObjectUtils.isEmpty(msgAlarm.getReleasedType())) {
                msgAlarmResp.setReleasedTypeName(msgAlarmResp.getReleasedType() == 1 ? "实报" : "误报");
            }
            msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            return AjaxResult.success(msgAlarmResp);
        } else {
            return success("无报警信息数据");
        }
    }

    @GetMapping(value = "/{id}")
    @Log(title = "报警信息管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getInfo(@PathVariable Long id) {
        MsgAlarm msgAlarm = msgAlarmService.getById(id);
        if (!ObjectUtils.isEmpty(msgAlarm)) {
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarm.setImageUrls(objects);
            if (StringUtils.isNotEmpty(msgAlarm.getReleasedBy())) {
                BaseUser user = baseUserService.getUser(msgAlarm.getReleasedBy());
                msgAlarm.setReleasedByName(user.getNickName());
            }
            if (!ObjectUtils.isEmpty(msgAlarm.getReleasedType())) {
                msgAlarm.setReleasedTypeName(msgAlarm.getReleasedType() == 1 ? "实报" : "误报");
            }
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            if (!ObjectUtils.isEmpty(baseCameraManagement.getEndMile())&&!ObjectUtils.isEmpty(baseCameraManagement.getBeginMile())) {
                String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
                String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
                String endMile = "[" + begin + "]-[" + end + "]";
                msgAlarmResp.setBeginMile(endMile);
            }
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
            msgAlarmResp.setDeptName(sysDept.getDeptName());
            String[] split1 = msgAlarm.getAlarmAttr().split(",");
            String stringBuilder = "画面,大小为" + split1[0] + "×" + split1[1] + "," +
                    "泥石流范围为左上角坐标(" + split1[2] + "," + split1[3] + ")" +
                    "大小" + split1[4] + "×" + split1[5];
            msgAlarmResp.setAlarmAttrName(stringBuilder);
            msgAlarmResp.setCardNum(baseCameraManagement.getCardNum());
            msgAlarmResp.setCameraSn(baseCameraManagement.getCameraSn());
            return AjaxResult.success(msgAlarmResp);
        } else {
            return success("无报警信息数据");
        }
    }

    // todo  需要测试 验证 是否合成视频
    @RequestMapping(value = "/updateIsLock")
    @Log(title = "报警信息管理", businessType = BusinessType.UPDATE)
    @Transactional
    public AjaxResult updateIsLock(@RequestParam(value = "id") Long id,
                                   @RequestParam(value = "releasedType", required = false) Integer releasedType,
                                   @RequestParam(value = "releasedReason", required = false) String releasedReason) {
        MsgAlarm msgAlarm = msgAlarmService.getById(id);
        if (msgAlarm.getIsLock() == 1) {
            return AjaxResult.error("501", "报警已解除");
        }
        msgAlarm.setIsLock(1);
        msgAlarm.setUpdateBy(getUserId());
        msgAlarm.setReleasedBy(getUserId());
        msgAlarm.setReleasedTime(new Date());
        if (!ObjectUtils.isEmpty(releasedReason)) {
            msgAlarm.setReleasedReason(releasedReason);
        }
        if (!ObjectUtils.isEmpty(releasedType)) {
            msgAlarm.setReleasedType(releasedType);
        }
        //添加报警视频url
        LambdaQueryWrapper<BaseCameraManagement> base = new LambdaQueryWrapper<>();
        base.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(base);
        String url = null;
        try {
            url = CameraUtil.historyPlayListStr(baseCameraManagement.getChannel(),
                    msgAlarm.getAlarmTime(),
                    msgAlarm.getReleasedTime(),
                    true);
        } catch (Exception exception) {
            log.error(exception.getMessage());
        }
        log.info("updateIsLock-------url" + url);
        msgAlarm.setReleasedUrl(url);
        log.info("updateIsLock----------msgAlarm" + msgAlarm);
        BaseCameraManagement bcm = msgAlarmService.getBaseCameraManagementByCamereCode(msgAlarm.getCameraCode());
        BaseCameraVO  baseCameraVO=new BaseCameraVO();
        baseCameraVO.setCameraCode(bcm.getCameraCode());
        baseCameraVO.setChannel(bcm.getChannel());
        redisService.removeFromSet(Constants.ALARM_SET,baseCameraVO);
        boolean b = msgAlarmService.updateById(msgAlarm);
        msgAlarmService.getNormalBaseCameraManagementsCache(Constants.ALARM_SET);
//        LambdaQueryWrapper<MsgAppPush> wrapper = new LambdaQueryWrapper<>();
//        wrapper.eq(MsgAppPush::getAlarmId, msgAlarm.getAlarmId());
//        MsgAppPush app = new MsgAppPush();
//        app.setStatus(1);
//        boolean update = msgAppPushService.update(app, wrapper);
        LambdaQueryWrapper<MsgWebPush> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MsgWebPush::getAlarmId, msgAlarm.getAlarmId());
        MsgWebPush web = new MsgWebPush();
        web.setOnline("1");
        boolean update1 = msgWebPushService.update(web, queryWrapper);

        if (b && update1) {
            return AjaxResult.success();
        } else {
            return AjaxResult.error();
        }
    }

    @GetMapping(value = "/alarmHistoricalAlarm/{alarmId}")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult alarmHistoricalAlarm(@PathVariable String alarmId) {
        QueryWrapper<MsgAlarm> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("alarm_id", alarmId);
        MsgAlarm one = msgAlarmService.getOne(queryWrapper);
        if (ObjectUtils.isEmpty(one)) {
            return error("报警表没有此报警唯一标识的报警数据");
        }
        QueryWrapper<MsgAlarmFrequency> wrapper = new QueryWrapper<>();
        wrapper.eq("alarm_id", alarmId);
        wrapper.orderByDesc("alarm_time");
        List<MsgAlarmFrequency> list = msgAlarmFrequencyService.list(wrapper);
        if (list.size() > 0) {
            if (one.getIsLock() == 1) {
                for (int i = 0; i < list.size(); i++) {
                    if (i == 0) {
                        list.get(i).setIsLock(1);
                    } else {
                        list.get(i).setIsLock(2);
                    }
                }
            } else {
                for (MsgAlarmFrequency msgAlarmFrequency : list) {
                    msgAlarmFrequency.setIsLock(2);
                }
            }
            return AjaxResult.success(list);
        } else {
            return error("报警频次表没有此报警唯一标识的报警数据");
        }
    }

    @GetMapping(value = "/videoHistoricalAlarm/{cameraCode}")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult videoHistoricalAlarm(@PathVariable String cameraCode) {
        QueryWrapper<MsgAlarm> wrapper = new QueryWrapper<>();
        wrapper.eq("camera_code", cameraCode);
        wrapper.orderByDesc("alarm_time");
        List<MsgAlarm> list = msgAlarmService.list(wrapper);
//        if (list.size() > 0) {
//            for (MsgAlarm msgAlarm : list) {
//                msgAlarm.setFailureToAlarm(1);
//            }
//            return AjaxResult.success(list);
//        } 
        return AjaxResult.success(list);
    }

    /**
     * web数据统计
     *
     * @param alarmStatisticResVo
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.OTHER)
    @PostMapping("/dataStatistic")
    public AjaxResult dataStatistic(@RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        alarmStatisticResVo = (AlarmStatisticResVo) dataScoreUtil.setDataScore(getUserId(), alarmStatisticResVo);
        IPage<AlarmStatisticVo> page = msgAlarmService.list(alarmStatisticResVo);
        page.setTotal(page.getTotal());
        page.setCurrent(page.getCurrent());
        page.setPages(page.getPages());
        if (!ObjectUtils.isEmpty(page) && page.getRecords().size() > 0) {
            List<AlarmStatisticVo> dto1 = page.getRecords().stream().map(o -> {
                if (!ObjectUtils.isEmpty(o.getAlarmMile())) {
                    o.setAlarmMile(AppendUtils.stringAppend(Integer.valueOf(o.getAlarmMile())));
                }
                if (StringUtils.isNotEmpty(o.getReleasedBy())) {
                    BaseUser buser = baseUserService.getUser(o.getReleasedBy());
                    o.setReleasedByName(buser.getNickName());
                }
                if (!ObjectUtils.isEmpty(o.getReleasedType())) {
                    o.setReleasedTypeName(o.getReleasedType() == 1 ? "实报" : "误报");
                }
                if (!ObjectUtils.isEmpty(o.getLineDir())) {
                    if (o.getLineDir() == 1) {
                        o.setLineDirName("上行");
                    } else if (o.getLineDir() == 2) {
                        o.setLineDirName( "下行");
                    } else if (o.getLineDir() == 3) {
                        o.setLineDirName( "上下行");
                    }
                }
                if (!ObjectUtils.isEmpty(o.getAlarmType())) {
                    o.setAlarmTypeName(dictDataService.selectDictLabel("sys_alarm_type", String.valueOf(o.getAlarmType())));
                }
                if (!ObjectUtils.isEmpty(o.getIsLock())) {
                    o.setIsLockName(o.getIsLock() == 1 ? "已解除" : "未解除");
                }
                if (!ObjectUtils.isEmpty(o.getAlarmAttr())) {
                    String[] split1 = o.getAlarmAttr().split(",");
                    String stringBuilder = "画面大小" + split1[0] + "×" + split1[1] + "," +
                            "泥石流范围为左上角坐标(" + split1[2] + "," + split1[3] + ")" +
                            "大小" + split1[4] + "×" + split1[5];
                    o.setAlarmAttrString(stringBuilder);
                }
                return o;
            }).collect(Collectors.toList());
                        page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }

    /**
     * web数据统计
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.OTHER)
    @PostMapping("/alarmMonthStatistic")
    public AjaxResult alarmMonthStatistic(@RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        alarmStatisticResVo = (AlarmStatisticResVo) dataScoreUtil.setDataScore(getUserId(), alarmStatisticResVo);
        List<Integer> numbs = new ArrayList<>();
        Integer listSize;
        List<AlarmStatisticMonthDto> list = null;
        if (ObjectUtils.isEmpty(alarmStatisticResVo.getCurrentMonth())) {
            list = msgAlarmService.getCount(alarmStatisticResVo);
            listSize = 12;
        } else {
            if (alarmStatisticResVo.getCurrentMonth().equals(2)) {//2
                listSize = 28;
            } else if (alarmStatisticResVo.getCurrentMonth().equals(4) || alarmStatisticResVo.getCurrentMonth().equals(6) ||
                    alarmStatisticResVo.getCurrentMonth().equals(9) || alarmStatisticResVo.getCurrentMonth().equals(11)) {//4,6,9,11
                listSize = 30;
            } else {//1,3,5,7,8,10,12
                listSize = 31;
            }
            list = msgAlarmService.getDayCount(alarmStatisticResVo);
        }
        for (int i = 1; i <= listSize; i++) {
            numbs.add(i);
        }

        List<Integer> list1 = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
            list.forEach(l -> {
                list1.add(l.getMonthsta());
            });
        }
        AlarmStatisticMonthDto dto;
        for (int i = 0; i < numbs.size(); i++) {
            if (!list1.contains(numbs.get(i))) {
                dto = new AlarmStatisticMonthDto();
                dto.setMonthsta(numbs.get(i));
                dto.setFrequency(0);
                list.add(dto);
            }
        }
        if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
            Collections.sort(list, new Comparator<AlarmStatisticMonthDto>() {
                @Override
                public int compare(AlarmStatisticMonthDto o1, AlarmStatisticMonthDto o2) {

                    return o1.getMonthsta().compareTo(o2.getMonthsta());
                }
            });
        }
        return AjaxResult.success(list);
    }


    /**
     * web数据统计 数据导出
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.EXPORT)
    @PostMapping("/exportDataStatistic")
    public void exportDataStatistic(HttpServletResponse response, @RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        alarmStatisticResVo = (AlarmStatisticResVo) dataScoreUtil.setDataScore(getUserId(), alarmStatisticResVo);
        try {
            List<AlarmStatisticDto> list = msgAlarmService.exportExcel(alarmStatisticResVo);
            if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
                list.forEach(l -> {
                    if (!ObjectUtils.isEmpty(l.getAlarmMile())) {
                        l.setAlarmMiles(AppendUtils.stringAppend(Integer.valueOf(l.getAlarmMile())));
                    }
                    if (StringUtils.isNotEmpty(l.getReleasedBy())) {
                        BaseUser buser = baseUserService.getUser(l.getReleasedBy());
                        l.setReleasedByName(buser.getNickName());
                    }
                    if (!ObjectUtils.isEmpty(l.getReleasedType())) {
                        l.setReleasedTypeName(l.getReleasedType() == 1 ? "实报" : "误报");
                    }
                    if (!ObjectUtils.isEmpty(l.getAlarmType())) {
                        String alarmType = l.getAlarmType();
                        String alarmTypeValue = dictDataService.selectDictLabel("sys_alarm_type", alarmType);
                        l.setAlarmTypeValue(alarmTypeValue);
                    }
                    if (!ObjectUtils.isEmpty(l.getLineDir())) {
                        String lineDir = l.getLineDir();
                        String lineDirValue = dictDataService.selectDictLabel("sys_line_dir", lineDir);
                        l.setLineDirValue(lineDirValue);
                    }
                    if (!ObjectUtils.isEmpty(l.getIsLock())) {
                        l.setIsLockName(l.getIsLock() == 1 ? "已解除" : "未解除");
                    }
                    if (!ObjectUtils.isEmpty(l.getAlarmAttr())) {
                        String[] split1 = l.getAlarmAttr().split(",");
                        String stringBuilder = "画面大小" + split1[0] + "×" + split1[1] + "," +
                                "泥石流范围为左上角坐标(" + split1[2] + "," + split1[3] + ")" +
                                "大小" + split1[4] + "×" + split1[5];
                        l.setAlarmAttrString(stringBuilder);
                    }
                });
            }

            ExcelUtil<AlarmStatisticDto> util = new ExcelUtil<>(AlarmStatisticDto.class);
            util.exportExcel(response, list, "报警数据详情");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
        }
    }

    /**
     * web数据统计 报警月统计数据导出
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.EXPORT)
    @PostMapping("/exportDataStatisticMonth")
    public void exportDataStatisticMonth(HttpServletResponse response, @RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        alarmStatisticResVo = (AlarmStatisticResVo) dataScoreUtil.setDataScore(getUserId(), alarmStatisticResVo);
        String sheetName = "报警年统计数据";
        List<Integer> numbs = new ArrayList<>();
        Integer listSize;
        List<AlarmStatisticMonthDto> list = null;
        if (ObjectUtils.isEmpty(alarmStatisticResVo.getCurrentMonth())) {
            list = msgAlarmService.getCount(alarmStatisticResVo);
            listSize = 12;
        } else {
            sheetName = "报警月统计数据";
            if (alarmStatisticResVo.getCurrentMonth().equals(2)) {//2
                listSize = 28;
            } else if (alarmStatisticResVo.getCurrentMonth().equals(4) || alarmStatisticResVo.getCurrentMonth().equals(6) ||
                    alarmStatisticResVo.getCurrentMonth().equals(9) || alarmStatisticResVo.getCurrentMonth().equals(11)) {//4,6,9,11
                listSize = 30;
            } else {//1,3,5,7,8,10,12
                listSize = 31;
            }
            list = msgAlarmService.getDayCount(alarmStatisticResVo);
        }
        if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
            for (int i = 1; i <= listSize; i++) {
                numbs.add(i);
            }
            List<Integer> list1 = new ArrayList<>();
            if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
                list.forEach(l -> {
                    list1.add(l.getMonthsta());
                });
            }
            AlarmStatisticMonthDto dto;
            for (int i = 0; i < numbs.size(); i++) {
                if (!list1.contains(numbs.get(i))) {
                    dto = new AlarmStatisticMonthDto();
                    String split = numbs.get(i).toString();
                    if (numbs.get(i) <= 9) {
                        split = "0" + numbs.get(i);
                    }
                    if (ObjectUtils.isEmpty(alarmStatisticResVo.getCurrentMonth())) {//年数据
                        dto.setMonthstaString(list.get(0).getMonthstaString().substring(0, 5) + split + "月");
                    } else {//月数据
                        dto.setMonthstaString(list.get(0).getMonthstaString().substring(0, 8) + split + "日");
                    }
                    dto.setMonthsta(numbs.get(i));
                    dto.setFrequency(0);
                    list.add(dto);
                }
            }
        }
        if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
            Collections.sort(list, new Comparator<AlarmStatisticMonthDto>() {
                @Override
                public int compare(AlarmStatisticMonthDto o1, AlarmStatisticMonthDto o2) {

                    return o1.getMonthsta().compareTo(o2.getMonthsta());
                }
            });
        }
        ExcelUtil<AlarmStatisticMonthDto> util = new ExcelUtil<>(AlarmStatisticMonthDto.class);
        util.exportExcel(response, list, sheetName);
    }


    @GetMapping(value = "/alarmStatistics")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult alarmStatistics() {
      return   msgAlarmService.alarmStatistics();
    }

    @GetMapping(value = "/alarmCountSort")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult alarmCountSort() {
        return   msgAlarmService.alarmCountSort();
    }

}

