package com.ozs.web.controller.shotschedule;

import com.ozs.common.constant.Constants;
import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.utils.DictUtils;
import com.ozs.common.utils.uuid.IdUtils;
import com.ozs.service.entity.vo.BaseCameraVO;
import com.ozs.service.service.BaseCameraManagementService;
import com.ozs.service.service.MsgAlarmService;
import com.ozs.service.service.RedisService;
import com.ozs.service.service.impl.CameraCaptureService;
import com.ozs.system.service.ISysDictTypeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Set;

/**
 * @author wyy
 * @subject
 * @creat 2024/3/1
 */
@Slf4j
@Service
public class TaskService {
    @Resource
    MsgAlarmService msgAlarmService;
    @Resource
    CameraCaptureService cameraCaptureService;
    @Value("${shot.urlAddress}")
    private String urlAddress;
    @Value("${shot.storeAddress}")
    private String storeAddress;
    @Autowired
    private ISysDictTypeService iSysDictTypeService;
    @Autowired
    private RedisService redisService;

    public void getNormalPicture() {
        String address = storeAddress;
        List<SysDictData> addressDataList = DictUtils.getDictCache("shot_address");
        if(CollectionUtils.isEmpty(addressDataList) || Objects.isNull(addressDataList.get(0))){
            addressDataList = iSysDictTypeService.selectDictDataByType("shot_address");
            DictUtils.setDictCache("shot_address",addressDataList);
        }
        if(!CollectionUtils.isEmpty(addressDataList) && Objects.nonNull(addressDataList.get(0))){
            address = addressDataList.get(0).getDictLabel();
        }
        boolean isHasKeyNormal = redisService.hasKey(Constants.NORMAL_SET);
        if(!isHasKeyNormal){
            msgAlarmService.initializationCameraCache();
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Set<BaseCameraVO> normalSet = redisService.getSetMembers(Constants.NORMAL_SET);
        if (log.isDebugEnabled()) {
            log.debug("正常摄像机数量:"+normalSet.size());
        }
        if (!CollectionUtils.isEmpty(normalSet)) {
            String finalAddress = address;
            normalSet.forEach(item -> {
                String uuid = IdUtils.fastSimpleUUID();
                if (!ObjectUtils.isEmpty(item.getCameraCode())) {
                    try {
                        if (log.isDebugEnabled()) {
                            log.debug("请求url======" + urlAddress + item.getCameraCode() + "/" + item.getChannel());
                        }
                        URL url = new URL(urlAddress + item.getCameraCode() + "/" + item.getChannel());
                        String dateString = sdf.format(new Date());
                        String fileName = finalAddress + "/" + "normal" + "/" + dateString + "/" + uuid + ".jpeg";
                        if (log.isDebugEnabled()) {
                            log.debug("fileName======" + fileName);
                            log.debug("正常摄像头截图开始");
                        }
                        cameraCaptureService.getCapture(url, fileName);
                        if (log.isDebugEnabled()) {
                            log.debug("正常摄像头截图结束");
                        }
                    } catch (Throwable e) {
                        log.error("正常摄像头截图异常",e);
                    }
                }
            });
        }
    }

    public void getAlarmPicture() {
        String address = storeAddress;
        List<SysDictData> addressDataList = DictUtils.getDictCache("shot_address");
        if(CollectionUtils.isEmpty(addressDataList) || Objects.isNull(addressDataList.get(0))){
            addressDataList = iSysDictTypeService.selectDictDataByType("shot_address");
            DictUtils.setDictCache("shot_address",addressDataList);
        }
        if(!CollectionUtils.isEmpty(addressDataList) && Objects.nonNull(addressDataList.get(0))){
            address = addressDataList.get(0).getDictLabel();
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        boolean isHasKeyAlarm = redisService.hasKey(Constants.ALARM_SET);
        if(!isHasKeyAlarm){
            msgAlarmService.initializationCameraCache();
        }
        Set<BaseCameraVO> alarmSet = redisService.getSetMembers(Constants.ALARM_SET);
        if (log.isDebugEnabled()) {
            log.debug("报警摄像机数量:"+alarmSet.size());
        }
        if(!CollectionUtils.isEmpty(alarmSet)){
            String finalAddress = address;
            alarmSet.forEach(item -> {
                String uuid = IdUtils.fastSimpleUUID();
                if(!ObjectUtils.isEmpty(item.getCameraCode())) {
                    try {
                        if (log.isDebugEnabled()) {
                            log.debug("请求url======" + urlAddress + item.getCameraCode() + "/" + item.getChannel());
                        }
                        URL url = new URL(urlAddress + item.getCameraCode() + "/" + item.getChannel());
                        String dateString = sdf.format(new Date());
                        String fileName = finalAddress + "/" + "alarm" + "/" + dateString + "/" + uuid + ".jpeg";
                        if (log.isDebugEnabled()) {
                            log.debug("fileName======" + fileName);
                            log.debug("报警摄像头截图开始");
                        }
                        cameraCaptureService.getCapture(url, fileName);
                        if (log.isDebugEnabled()) {
                            log.debug("报警摄像头截图结束");
                        }

                    } catch (Throwable e) {
                        log.error("报警摄像头截图异常",e);
                    }
                }
            });
        }
    }
}
