package com.ozs.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.core.domain.vo.SysDictTypeVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ozs.common.annotation.Log;
import com.ozs.common.constant.UserConstants;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDictType;
import com.ozs.common.core.page.TableDataInfo;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.system.service.ISysDictTypeService;

/**
 * 数据字典信息
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/dict/type")
public class SysDictTypeController extends BaseController {
    @Autowired
    private ISysDictTypeService dictTypeService;

//    @PreAuthorize("@ss.hasPermi('system:dict:list')")
//    @GetMapping("/list")
//    public TableDataInfo list(SysDictType dictType)
//    {
//        startPage();
//        List<SysDictType> list = dictTypeService.selectDictTypeList(dictType);
//        return getDataTable(list);
//    }

    /**
     * 获取字典类型列表
     *
     * @param sysDictTypeVo
     * @return
     */
    @PostMapping("/list")
    @PreAuthorize("@ss.hasPermi('system:dict:list')")
    public AjaxResult list(@RequestBody SysDictTypeVo sysDictTypeVo) {
        LambdaQueryWrapper<SysDictType> wrapper = new LambdaQueryWrapper<SysDictType>();
        if (!ObjectUtils.isEmpty(sysDictTypeVo.getDictId())) {
            wrapper.like(SysDictType::getDictId, sysDictTypeVo.getDictId());
        }
        if (!ObjectUtils.isEmpty(sysDictTypeVo.getDataType())) {
            wrapper.like(SysDictType::getDataType, sysDictTypeVo.getDataType());
        }
        if (!ObjectUtils.isEmpty(sysDictTypeVo.getDictName())) {
            wrapper.like(SysDictType::getDictName, sysDictTypeVo.getDictName());
        }
        if (!ObjectUtils.isEmpty(sysDictTypeVo.getDictType())) {
            wrapper.like(SysDictType::getDictType, sysDictTypeVo.getDictType());
        }
        if (!ObjectUtils.isEmpty(sysDictTypeVo.getStatus())) {
            wrapper.like(SysDictType::getStatus, sysDictTypeVo.getStatus());
        }
        IPage<SysDictType> page = dictTypeService.page(new Page<>(sysDictTypeVo.getPageNum(), sysDictTypeVo.getPageSize()), wrapper);
        return AjaxResult.success(page);
    }

    @Log(title = "字典类型", businessType = BusinessType.EXPORT)
    @PreAuthorize("@ss.hasPermi('system:dict:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SysDictType dictType) {
        List<SysDictType> list = dictTypeService.selectDictTypeList(dictType);
        ExcelUtil<SysDictType> util = new ExcelUtil<SysDictType>(SysDictType.class);
        util.exportExcel(response, list, "字典类型");
    }

    /**
     * 查询字典类型详细
     */
    @PreAuthorize("@ss.hasPermi('system:dict:query')")
    @GetMapping(value = "/{dictId}")
    public AjaxResult getInfo(@PathVariable Long dictId)
    {
        return success(dictTypeService.selectDictTypeById(dictId));
    }

    /**
     * 新增字典类型
     */
    @PreAuthorize("@ss.hasPermi('system:dict:add')")
    @Log(title = "字典类型", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysDictType dict)
    {
        if (UserConstants.NOT_UNIQUE.equals(dictTypeService.checkDictTypeUnique(dict))) {
            return error("新增字典'" + dict.getDictType() + "'失败，字典类型已存在");
        }
        if (UserConstants.NOT_UNIQUE.equals(dictTypeService.checkDictNameUnique(dict))) {
            return error("新增字典'" + dict.getDictName() + "'失败，字典名称已存在");
        }
        dict.setCreateBy(getUsername());
        return toAjax(dictTypeService.insertDictType(dict));
    }

    /**
     * 修改字典类型
     */
    @PreAuthorize("@ss.hasPermi('system:dict:edit')")
    @Log(title = "字典类型", businessType = BusinessType.UPDATE)
    @RequestMapping("/update")
    public AjaxResult edit(@Validated @RequestBody SysDictType dict)
    {
        if (UserConstants.NOT_UNIQUE.equals(dictTypeService.checkDictTypeUnique(dict))) {
            return error("修改字典'" + dict.getDictType() + "'失败，字典类型已存在");
        }
        if (UserConstants.NOT_UNIQUE.equals(dictTypeService.checkDictNameUnique(dict))) {
            return error("修改字典'" + dict.getDictName() + "'失败，字典名称已存在");
        }
        dict.setUpdateBy(getUsername());
        return toAjax(dictTypeService.updateDictType(dict));
    }

    /**
     * 删除字典类型
     */
    @PreAuthorize("@ss.hasPermi('system:dict:remove')")
    @Log(title = "字典类型", businessType = BusinessType.DELETE)
    @RequestMapping("/remove/{dictIds}")
    public AjaxResult remove(@PathVariable Long[] dictIds)
    {
        dictTypeService.deleteDictTypeByIds(dictIds);
        return success();
    }

    /**
     * 刷新字典缓存
     */
    @PreAuthorize("@ss.hasPermi('system:dict:remove')")
    @Log(title = "字典类型", businessType = BusinessType.CLEAN)
    @GetMapping("/refreshCache")
    public AjaxResult refreshCache()
    {
        dictTypeService.resetDictCache();
        return success();
    }

    /**
     * 获取字典选择框列表
     */
    @GetMapping("/optionselect")
    public AjaxResult optionselect(Integer dataType) {
        List<SysDictType> dictTypes = dictTypeService.selectDictTypeAll(dataType);
        return success(dictTypes);
    }

    /**
     * 获取字典类型名称
     */
    @PostMapping("/selectList")
    public AjaxResult selectList() {
        List<SysDictType> dictTypes = dictTypeService.list();
        return success(dictTypes);
    }
}
